@@ -20,6 +20,7 @@ Command-line interface to the OpenStack Cinder API.
 from __future__ import print_function
 
 import argparse
+import collections
 import getpass
 import logging
 import sys
@@ -137,21 +138,6 @@ class OpenStackCinderShell(object):
                                               default=False),
                             help=_('Shows debugging output.'))
 
-        parser.add_argument('--os-auth-system',
-                            metavar='<os-auth-system>',
-                            dest='os_auth_type',
-                            default=utils.env('OS_AUTH_SYSTEM'),
-                            help=_('DEPRECATED! Use --os-auth-type. '
-                                   'Defaults to env[OS_AUTH_SYSTEM].'))
-        parser.add_argument('--os_auth_system',
-                            help=argparse.SUPPRESS)
-        parser.add_argument('--os-auth-type',
-                            metavar='<os-auth-type>',
-                            dest='os_auth_type',
-                            default=utils.env('OS_AUTH_TYPE'),
-                            help=_('Defaults to env[OS_AUTH_TYPE].'))
-        parser.add_argument('--os_auth_type',
-                            help=argparse.SUPPRESS)
         parser.add_argument('--service-type',
                             metavar='<service-type>',
                             help=_('Service type. '
@@ -252,11 +238,16 @@ class OpenStackCinderShell(object):
         return parser
 
     def _append_global_identity_args(self, parser):
-        # FIXME(bklei): these are global identity (Keystone) arguments which
-        # should be consistent and shared by all service clients. Therefore,
-        # they should be provided by python-keystoneclient. We will need to
-        # refactor this code once this functionality is available in
-        # python-keystoneclient.
+        loading.register_session_argparse_arguments(parser)
+
+        # Use "password" auth plugin as default and keep the explicit
+        # "--os-token" arguments below for backward compatibility.
+        default_auth_plugin = 'password'
+
+        # Passing [] to loading.register_auth_argparse_arguments to avoid
+        # the auth_type being overridden by the command line.
+        loading.register_auth_argparse_arguments(
+            parser, [], default=default_auth_plugin)
 
         parser.add_argument(
             '--os-auth-strategy', metavar='<auth-strategy>',
@@ -268,128 +259,88 @@ class OpenStackCinderShell(object):
             '--os_auth_strategy',
             help=argparse.SUPPRESS)
 
-        parser.add_argument('--os-username',
-                            metavar='<auth-user-name>',
-                            default=utils.env('OS_USERNAME',
-                                              'CINDER_USERNAME'),
-                            help=_('OpenStack user name. '
-                            'Default=env[OS_USERNAME].'))
+        # Change os_auth_type default value defined by
+        # register_auth_argparse_arguments to be backward compatible
+        # with OS_AUTH_SYSTEM.
+        env_plugin = utils.env('OS_AUTH_TYPE',
+                               'OS_AUTH_PLUGIN',
+                               'OS_AUTH_SYSTEM')
+        parser.set_defaults(os_auth_type=env_plugin)
+        parser.add_argument('--os_auth_type',
+                            help=argparse.SUPPRESS)
+
+        parser.add_argument('--os-auth-system',
+                            metavar='<os-auth-system>',
+                            dest='os_auth_type',
+                            default=env_plugin,
+                            help=_('DEPRECATED! Use --os-auth-type. '
+                                   'Defaults to env[OS_AUTH_SYSTEM].'))
+        parser.add_argument('--os_auth_system',
+                            help=argparse.SUPPRESS)
+
+        parser.set_defaults(os_username=utils.env('OS_USERNAME',
+                                                  'CINDER_USERNAME'))
         parser.add_argument('--os_username',
                             help=argparse.SUPPRESS)
 
-        parser.add_argument('--os-password',
-                            metavar='<auth-password>',
-                            default=utils.env('OS_PASSWORD',
-                                              'CINDER_PASSWORD'),
-                            help=_('Password for OpenStack user. '
-                            'Default=env[OS_PASSWORD].'))
+        parser.set_defaults(os_password=utils.env('OS_PASSWORD',
+                                                  'CINDER_PASSWORD'))
         parser.add_argument('--os_password',
                             help=argparse.SUPPRESS)
 
-        parser.add_argument('--os-tenant-name',
-                            metavar='<auth-tenant-name>',
-                            default=utils.env('OS_TENANT_NAME',
-                                              'OS_PROJECT_NAME',
-                                              'CINDER_PROJECT_ID'),
-                            help=_('Tenant name. '
-                            'Default=env[OS_TENANT_NAME].'))
+        # tenant_name is deprecated by project_name in keystoneauth
+        parser.set_defaults(os_project_name=utils.env('OS_PROJECT_NAME',
+                                                      'OS_TENANT_NAME',
+                                                      'CINDER_PROJECT_ID'))
         parser.add_argument('--os_tenant_name',
+                            dest='os_project_name',
                             help=argparse.SUPPRESS)
-
-        parser.add_argument('--os-tenant-id',
-                            metavar='<auth-tenant-id>',
-                            default=utils.env('OS_TENANT_ID',
-                                              'OS_PROJECT_ID',
-                                              'CINDER_TENANT_ID'),
-                            help=_('ID for the tenant. '
-                            'Default=env[OS_TENANT_ID].'))
-        parser.add_argument('--os_tenant_id',
-                            help=argparse.SUPPRESS)
-
-        parser.add_argument('--os-auth-url',
-                            metavar='<auth-url>',
-                            default=utils.env('OS_AUTH_URL',
-                                              'CINDER_URL'),
-                            help=_('URL for the authentication service. '
-                            'Default=env[OS_AUTH_URL].'))
-        parser.add_argument('--os_auth_url',
-                            help=argparse.SUPPRESS)
-
-        parser.add_argument(
-            '--os-user-id', metavar='<auth-user-id>',
-            default=utils.env('OS_USER_ID'),
-            help=_('Authentication user ID (Env: OS_USER_ID).'))
-
-        parser.add_argument(
-            '--os_user_id',
-            help=argparse.SUPPRESS)
-
-        parser.add_argument(
-            '--os-user-domain-id',
-            metavar='<auth-user-domain-id>',
-            default=utils.env('OS_USER_DOMAIN_ID'),
-            help=_('OpenStack user domain ID. '
-            'Defaults to env[OS_USER_DOMAIN_ID].'))
-
-        parser.add_argument(
-            '--os_user_domain_id',
-            help=argparse.SUPPRESS)
-
-        parser.add_argument(
-            '--os-user-domain-name',
-            metavar='<auth-user-domain-name>',
-            default=utils.env('OS_USER_DOMAIN_NAME'),
-            help=_('OpenStack user domain name. '
-                 'Defaults to env[OS_USER_DOMAIN_NAME].'))
-
-        parser.add_argument(
-            '--os_user_domain_name',
-            help=argparse.SUPPRESS)
-
-        parser.add_argument(
-            '--os-project-id',
-            metavar='<auth-project-id>',
-            default=utils.env('OS_PROJECT_ID'),
-            help=_('Another way to specify tenant ID. '
-            'This option is mutually exclusive with '
-            ' --os-tenant-id. '
-            'Defaults to env[OS_PROJECT_ID].'))
-
-        parser.add_argument(
-            '--os_project_id',
-            help=argparse.SUPPRESS)
-
-        parser.add_argument(
-            '--os-project-name',
-            metavar='<auth-project-name>',
-            default=utils.env('OS_PROJECT_NAME'),
-            help=_('Another way to specify tenant name. '
-                 'This option is mutually exclusive with '
-                 ' --os-tenant-name. '
-                 'Defaults to env[OS_PROJECT_NAME].'))
-
         parser.add_argument(
             '--os_project_name',
             help=argparse.SUPPRESS)
 
+        # tenant_id is deprecated by project_id in keystoneauth
+        parser.set_defaults(os_project_id=utils.env('OS_PROJECT_ID',
+                                                    'OS_TENANT_ID',
+                                                    'CINDER_TENANT_ID'))
+        parser.add_argument('--os_tenant_id',
+                            dest='os_project_id',
+                            help=argparse.SUPPRESS)
         parser.add_argument(
-            '--os-project-domain-id',
-            metavar='<auth-project-domain-id>',
-            default=utils.env('OS_PROJECT_DOMAIN_ID'),
-            help=_('Defaults to env[OS_PROJECT_DOMAIN_ID].'))
+            '--os_project_id',
+            help=argparse.SUPPRESS)
 
+        parser.set_defaults(os_auth_url=utils.env('OS_AUTH_URL',
+                                                  'CINDER_URL'))
+        parser.add_argument('--os_auth_url',
+                            help=argparse.SUPPRESS)
+
+        parser.set_defaults(os_user_id=utils.env('OS_USER_ID'))
         parser.add_argument(
-            '--os-project-domain-name',
-            metavar='<auth-project-domain-name>',
-            default=utils.env('OS_PROJECT_DOMAIN_NAME'),
-            help=_('Defaults to env[OS_PROJECT_DOMAIN_NAME].'))
+            '--os_user_id',
+            help=argparse.SUPPRESS)
 
-        parser.add_argument('--os-region-name',
-                            metavar='<region-name>',
-                            default=utils.env('OS_REGION_NAME',
-                                              'CINDER_REGION_NAME'),
-                            help=_('Region name. '
-                            'Default=env[OS_REGION_NAME].'))
+        parser.set_defaults(
+            os_user_domain_id=utils.env('OS_USER_DOMAIN_ID'))
+        parser.add_argument(
+            '--os_user_domain_id',
+            help=argparse.SUPPRESS)
+
+        parser.set_defaults(
+            os_user_domain_name=utils.env('OS_USER_DOMAIN_NAME'))
+        parser.add_argument(
+            '--os_user_domain_name',
+            help=argparse.SUPPRESS)
+
+        parser.set_defaults(
+            os_project_domain_id=utils.env('OS_PROJECT_DOMAIN_ID'))
+
+        parser.set_defaults(
+            os_project_domain_name=utils.env('OS_PROJECT_DOMAIN_NAME'))
+
+        parser.set_defaults(
+            os_region_name=utils.env('OS_REGION_NAME',
+                                     'CINDER_REGION_NAME'))
         parser.add_argument('--os_region_name',
                             help=argparse.SUPPRESS)
 
@@ -409,8 +360,6 @@ class OpenStackCinderShell(object):
             '--os_url',
             help=argparse.SUPPRESS)
 
-        # Register the CLI arguments that have moved to the session object.
-        loading.register_session_argparse_arguments(parser)
         parser.set_defaults(insecure=utils.env('CINDERCLIENT_INSECURE',
                                                default=False))
 
@@ -490,6 +439,7 @@ class OpenStackCinderShell(object):
                 action_help = desc.strip().split('\n')[0]
                 action_help += additional_msg
 
+            exclusive_args = getattr(callback, 'exclusive_args', {})
             arguments = getattr(callback, 'arguments', [])
 
             subparser = subparsers.add_parser(
@@ -504,41 +454,59 @@ class OpenStackCinderShell(object):
                                    help=argparse.SUPPRESS,)
 
             self.subcommands[command] = subparser
-
-            # NOTE(ntpttr): We get a counter for each argument in this
-            # command here because during the microversion check we only
-            # want to raise an exception if no version of the argument
-            # matches the current microversion. The exception will only
-            # be raised after the last instance of a particular argument
-            # fails the check.
-            arg_counter = dict()
-            for (args, kwargs) in arguments:
-                arg_counter[args[0]] = arg_counter.get(args[0], 0) + 1
-
-            for (args, kwargs) in arguments:
-                start_version = kwargs.get("start_version", None)
-                start_version = api_versions.APIVersion(start_version)
-                end_version = kwargs.get('end_version', None)
-                end_version = api_versions.APIVersion(end_version)
-                if do_help and (start_version or end_version):
-                    kwargs["help"] = kwargs.get("help", "") + (
-                        self._build_versioned_help_message(start_version,
-                                                           end_version))
-                if not version.matches(start_version, end_version):
-                    if args[0] in input_args and command == input_args[0]:
-                        if arg_counter[args[0]] == 1:
-                            # This is the last version of this argument,
-                            # raise the exception.
-                            raise exc.UnsupportedAttribute(args[0],
-                                start_version, end_version)
-                        arg_counter[args[0]] -= 1
-                    continue
-                kw = kwargs.copy()
-                kw.pop("start_version", None)
-                kw.pop("end_version", None)
-                subparser.add_argument(*args, **kw)
+            self._add_subparser_args(subparser, arguments, version, do_help,
+                                     input_args, command)
+            self._add_subparser_exclusive_args(subparser, exclusive_args,
+                                               version, do_help, input_args,
+                                               command)
             subparser.set_defaults(func=callback)
 
+    def _add_subparser_args(self, subparser, arguments, version, do_help,
+                            input_args, command):
+        # NOTE(ntpttr): We get a counter for each argument in this
+        # command here because during the microversion check we only
+        # want to raise an exception if no version of the argument
+        # matches the current microversion. The exception will only
+        # be raised after the last instance of a particular argument
+        # fails the check.
+        arg_counter = collections.defaultdict(int)
+        for (args, kwargs) in arguments:
+            arg_counter[args[0]] += 1
+
+        for (args, kwargs) in arguments:
+            start_version = kwargs.get("start_version", None)
+            start_version = api_versions.APIVersion(start_version)
+            end_version = kwargs.get('end_version', None)
+            end_version = api_versions.APIVersion(end_version)
+            if do_help and (start_version or end_version):
+                kwargs["help"] = kwargs.get("help", "") + (
+                    self._build_versioned_help_message(start_version,
+                                                       end_version))
+            if not version.matches(start_version, end_version):
+                if args[0] in input_args and command == input_args[0]:
+                    if arg_counter[args[0]] == 1:
+                        # This is the last version of this argument,
+                        # raise the exception.
+                        raise exc.UnsupportedAttribute(args[0],
+                            start_version, end_version)
+                    arg_counter[args[0]] -= 1
+                continue
+            kw = kwargs.copy()
+            kw.pop("start_version", None)
+            kw.pop("end_version", None)
+            subparser.add_argument(*args, **kw)
+
+    def _add_subparser_exclusive_args(self, subparser, exclusive_args,
+                                      version, do_help, input_args, command):
+        for group_name, arguments in exclusive_args.items():
+            if group_name == '__required__':
+                continue
+            required = exclusive_args['__required__'][group_name]
+            exclusive_group = subparser.add_mutually_exclusive_group(
+                required=required)
+            self._add_subparser_args(exclusive_group, arguments,
+                                     version, do_help, input_args, command)
+
     def setup_debugging(self, debug):
         if not debug:
             return
@@ -608,6 +576,12 @@ class OpenStackCinderShell(object):
                                                        do_help, args)
         self.parser = subcommand_parser
 
+        if argv and len(argv) > 1 and '--help' in argv:
+            argv = [x for x in argv if x != '--help']
+            if argv[0] in self.subcommands:
+                self.subcommands[argv[0]].print_help()
+                return 0
+
         if options.help or not argv:
             subcommand_parser.print_help()
             return 0
@@ -624,13 +598,13 @@ class OpenStackCinderShell(object):
             self.do_bash_completion(args)
             return 0
 
-        (os_username, os_password, os_tenant_name, os_auth_url,
-         os_region_name, os_tenant_id, endpoint_type,
+        (os_username, os_password, os_project_name, os_auth_url,
+         os_region_name, os_project_id, endpoint_type,
          service_type, service_name, volume_service_name, os_endpoint,
          cacert, os_auth_type) = (
              args.os_username, args.os_password,
-             args.os_tenant_name, args.os_auth_url,
-             args.os_region_name, args.os_tenant_id,
+             args.os_project_name, args.os_auth_url,
+             args.os_region_name, args.os_project_id,
              args.os_endpoint_type,
              args.service_type, args.service_name,
              args.volume_service_name,
@@ -653,12 +627,11 @@ class OpenStackCinderShell(object):
         # for os_username or os_password but for compatibility it is not.
 
         # V3 stuff
-        project_info_provided = ((self.options.os_tenant_name or
-                                  self.options.os_tenant_id) or
-                                 (self.options.os_project_name and
+        project_info_provided = ((self.options.os_project_name and
                                   (self.options.os_project_domain_name or
                                    self.options.os_project_domain_id)) or
-                                 self.options.os_project_id)
+                                 self.options.os_project_id or
+                                 self.options.os_project_name)
 
         # NOTE(e0ne): if auth_session exists it means auth plugin created
         # session and we don't need to check for password and other
@@ -730,9 +703,9 @@ class OpenStackCinderShell(object):
 
         self.cs = client.Client(
             api_version, os_username,
-            os_password, os_tenant_name, os_auth_url,
+            os_password, os_project_name, os_auth_url,
             region_name=os_region_name,
-            tenant_id=os_tenant_id,
+            tenant_id=os_project_id,
             endpoint_type=endpoint_type,
             extensions=self.extensions,
             service_type=service_type,
@@ -830,9 +803,8 @@ class OpenStackCinderShell(object):
 
         username = self.options.os_username
         password = self.options.os_password
-        tenant_id = self.options.os_tenant_id or self.options.os_project_id
-        tenant_name = (self.options.os_tenant_name or
-                       self.options.os_project_name)
+        tenant_id = self.options.os_project_id
+        tenant_name = self.options.os_project_name
 
         return v2_auth.Password(
             v2_auth_url,
@@ -848,9 +820,8 @@ class OpenStackCinderShell(object):
         user_domain_name = self.options.os_user_domain_name
         user_domain_id = self.options.os_user_domain_id
         password = self.options.os_password
-        project_id = self.options.os_project_id or self.options.os_tenant_id
-        project_name = (self.options.os_project_name or
-                        self.options.os_tenant_name)
+        project_id = self.options.os_project_id
+        project_name = self.options.os_project_name
         project_domain_name = self.options.os_project_domain_name
         project_domain_id = self.options.os_project_domain_id
 
