@@ -21,13 +21,11 @@ from cinderclient import api_versions
 from cinderclient import exceptions
 from cinderclient.tests.unit import utils
 from cinderclient.tests.unit.v3 import fakes
-from cinderclient.v3 import volumes
 from cinderclient.v3 import volume_snapshots
+from cinderclient.v3 import volumes
 
 from six.moves.urllib import parse
 
-cs = fakes.FakeClient()
-
 
 @ddt.ddt
 class VolumesTest(utils.TestCase):
@@ -73,6 +71,7 @@ class VolumesTest(utils.TestCase):
                               fake_volume.revert_to_snapshot, fake_snapshot)
 
     def test_create_volume(self):
+        cs = fakes.FakeClient(api_versions.APIVersion('3.13'))
         vol = cs.volumes.create(1, group_id='1234', volume_type='5678')
         expected = {'volume': {'status': 'creating',
                                'description': None,
@@ -90,7 +89,8 @@ class VolumesTest(utils.TestCase):
                                'source_replica': None,
                                'consistencygroup_id': None,
                                'multiattach': False,
-                               'group_id': '1234'}}
+                               'group_id': '1234',
+                               'backup_id': None}}
         cs.assert_called('POST', '/volumes', body=expected)
         self._assert_request_id(vol)
 
@@ -102,6 +102,16 @@ class VolumesTest(utils.TestCase):
         cs.volumes.summary(all_tenants=all_tenants)
         cs.assert_called('GET', url)
 
+    def test_volume_manage_cluster(self):
+        cs = fakes.FakeClient(api_versions.APIVersion('3.16'))
+        vol = cs.volumes.manage(None, {'k': 'v'}, cluster='cluster1')
+        expected = {'host': None, 'name': None, 'availability_zone': None,
+                    'description': None, 'metadata': None, 'ref': {'k': 'v'},
+                    'volume_type': None, 'bootable': False,
+                    'cluster': 'cluster1'}
+        cs.assert_called('POST', '/os-volume-manage', {'volume': expected})
+        self._assert_request_id(vol)
+
     def test_volume_list_manageable(self):
         cs = fakes.FakeClient(api_versions.APIVersion('3.8'))
         cs.volumes.list_manageable('host1', detailed=False)
@@ -145,3 +155,37 @@ class VolumesTest(utils.TestCase):
             request_url = '/scheduler-stats/get_pools?detail=True&name=pool1'
         cs.assert_called('GET', request_url)
         self._assert_request_id(vol)
+
+    def test_migrate_host(self):
+        cs = fakes.FakeClient(api_versions.APIVersion('3.0'))
+        v = cs.volumes.get('1234')
+        self._assert_request_id(v)
+        vol = cs.volumes.migrate_volume(v, 'host_dest', False, False)
+        cs.assert_called('POST', '/volumes/1234/action',
+                         {'os-migrate_volume': {'host': 'host_dest',
+                                                'force_host_copy': False,
+                                                'lock_volume': False}})
+        self._assert_request_id(vol)
+
+    def test_migrate_with_lock_volume(self):
+        cs = fakes.FakeClient(api_versions.APIVersion('3.0'))
+        v = cs.volumes.get('1234')
+        self._assert_request_id(v)
+        vol = cs.volumes.migrate_volume(v, 'dest', False, True)
+        cs.assert_called('POST', '/volumes/1234/action',
+                         {'os-migrate_volume': {'host': 'dest',
+                                                'force_host_copy': False,
+                                                'lock_volume': True}})
+        self._assert_request_id(vol)
+
+    def test_migrate_cluster(self):
+        cs = fakes.FakeClient(api_versions.APIVersion('3.16'))
+        v = cs.volumes.get('fake')
+        self._assert_request_id(v)
+        vol = cs.volumes.migrate_volume(v, 'host_dest', False, False,
+                                        'cluster_dest')
+        cs.assert_called('POST', '/volumes/fake/action',
+                         {'os-migrate_volume': {'cluster': 'cluster_dest',
+                                                'force_host_copy': False,
+                                                'lock_volume': False}})
+        self._assert_request_id(vol)
